@@ -31,6 +31,8 @@ class CommandCompletion(Completion):
                                 and isinstance(targetpanel, cmd.ACCEPT_IN)]
 
         if len(parts) > 1 or text.endswith(' '):
+            if text.endswith(' '):
+                parts.append('')
             cmd = resolve_command(parts[0])
             if cmd is not None and isinstance(targetpanel, cmd.ACCEPT_IN):
                 alternatives = cmd().completion_options(self.app.make_context(), *parts[1:])
@@ -174,13 +176,20 @@ def expand_part(app, part):
      - '%f' to [full path of selected item]
      - '%s' to [full path of selected item 1, full path of selected item 2, ...]
      - '%p' to [full path of folder of selected item]
+     - '%P' to [full path of the current folder] (only in file panel)
     """
     if part == '%n':
+        if app.current_panel.selected_path is None:
+            raise RuntimeError("Nothing selected")
         return [str(app.current_panel.selected_path.name)]
     if part == '%f':
         return [str(app.current_panel.selected_path)]
     if part == '%s':
         return [str(p) for p in app.current_panel.selected_paths]
     if part == '%p':
+        if app.current_panel.selected_path is None:
+            raise RuntimeError("Nothing selected")
         return [str(app.current_panel.selected_path.parent)]
+    if part == '%P' and hasattr(app.current_panel, 'path'):
+        return [str(app.current_panel.path)]
     return [part]
