@@ -2,7 +2,7 @@ import pathlib
 import tempfile
 import subprocess
 import string
-import os
+import datetime
 
 from cursedspace import ShellContext, InputLine
 
@@ -693,7 +693,11 @@ class RunIndexers(Command):
         context.application.paint(True)
 
     @staticmethod
-    def run_indexers(blocker, application, panel, path):
+    def run_indexers(blocker, application, panel, path, force=False):
+        """Run the indexers on the file at ``path``
+
+        ``force`` may be set to ``True`` to enforce re-indexing
+        """
         blocker.title(f"Running indexers on {path.name}")
         logger.debug("Running indexers on %s", path.name)
 
@@ -704,10 +708,16 @@ class RunIndexers(Command):
         else:
             paths = [path]
 
+        last_modified = {}
+        if force:
+            last_modified = {p: datetime.datetime.max
+                             for p in paths}
+
         results = index_files(paths,
                               application.metaindexconf,
                               ocr=metaindex.ocr.Dummy(),
-                              fulltext=False)
+                              fulltext=False,
+                              last_modified=last_modified)
         if len(results) == 0:
             return
 
