@@ -4,6 +4,7 @@ import tempfile
 import subprocess
 
 from metaindex import shared
+from metaindex import stores
 from metaindex.ocr import Dummy, TesseractOCR
 from metaindex.humanizer import humanize
 from metaindex.shared import DUBLINCORE_TAGS
@@ -16,7 +17,7 @@ from metaindexmanager import clipboard
 from metaindexmanager import command
 from metaindexmanager.shared import DICTIONARY_SCOPE
 from metaindexmanager.utils import logger, first_line
-from metaindexmanager.panel import ListPanel
+from metaindexmanager.panel import ListPanel, register
 from metaindexmanager.docpanel import DocPanel
 from metaindexmanager.filepanel import FilePanel
 from metaindexmanager.detailpanel import DetailPanel
@@ -25,6 +26,7 @@ from metaindexmanager.detailpanel import DetailPanel
 Change = collections.namedtuple("Change", ['index', 'new_value', 'prefix', 'tag', 'old_value'])
 Insert = collections.namedtuple("Insert", ['prefix', 'tag', 'value'])
 Delete = collections.namedtuple("Delete", ['index', 'prefix', 'tag', 'value'])
+AddToAll = collections.namedtuple("AddToAll", ['index', 'prefix', 'tag', 'value'])
 GroupedChange = collections.namedtuple("GroupedChange", ['changes'])
 
 
@@ -46,11 +48,13 @@ class Header(LineBase):
         return self.group.title()
 
 class Line(LineBase):
-    def __init__(self, group, prefix, tag, value):
+    def __init__(self, group, prefix, tag, value, partial):
         super().__init__(group)
         self.prefix = prefix
         self.tag = tag
         self.value = value
+        # whether or not this value exists in all files that are being edited
+        self.partial = partial
 
     def comparator(self):
         return super().comparator() + [self.tag.lower(), str(self.value).lower()]
@@ -157,6 +161,7 @@ class EditorLine(InputLine):
         self.parent.paint_item(self.parent.cursor)
 
 
+@register
 class EditorPanel(ListPanel):
     """Metadata editing panel"""
     SCOPE = 'editor'
@@ -168,9 +173,9 @@ class EditorPanel(ListPanel):
     CONFIG_TAGS = 'tags'
     CONFIG_TAGS_DEFAULT = ','.join({'*'} | DUBLINCORE_TAGS)
 
-    def __init__(self, filepath, *args, **kwargs):
+    def __init__(self, filepaths, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.item = filepath
+        self.files = filepaths
         self.columns = []
         # the history of changes
         self.changes = []
@@ -180,7 +185,10 @@ class EditorPanel(ListPanel):
         # list of items directly after reload
         self.unchanged_items = []
         self.editor = None
-        self.metadata = shared.CacheEntry(filepath)
+        # the merged metadata of all edited files
+        self.metadata = shared.CacheEntry(self.files[0])
+        # the original metadata extracted from sidecars per file
+        self.meta_per_file = {}
 
         self.auto_index_if_not_in_cache = True
 
@@ -198,15 +206,15 @@ class EditorPanel(ListPanel):
 
     @property
     def selected_path(self):
-        return self.item
+        return self.files[0]
 
     @property
     def selected_paths(self):
-        return [self.item]
+        return self.files
 
     @property
     def selected_item(self):
-        return str(self.item)
+        return str(self.files[0])
 
     @property
     def selected_items(self):
@@ -240,14 +248,14 @@ class EditorPanel(ListPanel):
         self.changed(grouped)
 
     def open_selected(self):
-        if self.item is None:
+        if len(self.files) != 1:
             return
-        self.app.open_file(self.item)
+        self.app.open_file(self.files[0])
 
     def open_selected_with(self, cmd):
-        if self.item is None:
+        if len(self.files) != 1:
             return
-        self.app.open_with(self.item, cmd)
+        self.app.open_with(self.files[0], cmd)
 
     def changed(self, change):
         logger.debug(f"Added change to stack: {change}")
@@ -270,7 +278,7 @@ class EditorPanel(ListPanel):
             self.win.noutrefresh()
 
     def title(self):
-        title = str(self.item)
+        title = " ".join([str(f) for f in self.files])
         if self.change_ptr > 0:
             title = "[Modified] " + title
         return title
@@ -282,6 +290,7 @@ class EditorPanel(ListPanel):
         else:
             self.win.move(y, x+self.columns[0])
             self.win.noutrefresh()
+        return y, x
 
     def multiline_edit(self):
         logger.debug(f"start multi-line editing {self.selected_line}")
@@ -384,11 +393,12 @@ class EditorPanel(ListPanel):
         if blocker is not None:
             blocker.title(f"Saving changes to {self.selected_path.name}")
 
-        self._apply_changes_to(self.selected_path)
+        for target in self.files:
+            self._apply_changes_to(target)
 
         # reload the cache
         with ShellContext(self.app.screen):
-            self.app.cache.refresh(self.selected_path)
+            self.app.cache.refresh(self.selected_paths)
             self.app.cache.wait_for_reload()
         self.app.paint(True)
 
@@ -396,43 +406,41 @@ class EditorPanel(ListPanel):
         self.reset()
 
     def _apply_changes_to(self, targetfile):
-        # Read in the current sidecar file, if it exists
-        collection_extra = None
-        sidecar_file, is_collection, store = \
-            self.app.metaindexconf.resolve_sidecar_for(targetfile)
+        miconf = self.app.metaindexconf
+        logger.debug(f"Applying metadata changes to {targetfile}")
 
-        if sidecar_file is None:
-            self.app.error("No usable metadata storage available")
-            return
+        # preferred sidecar store suffices, in order
+        preferred_sidecar_exts = [s.SUFFIX
+                                  for s in miconf.get_preferred_sidecar_stores()]
 
-        if sidecar_file.exists():
+        # existing direct sidecars
+        existing_sidecars = []
+
+        # the collection sidecar files that need to have their specific
+        # entries for targetfile removed
+        collections_to_touch = set()
+
+        # 1. read metadata from all sidecar files
+        metadata = shared.CacheEntry(targetfile)
+        for sidecar, is_collection in miconf.find_all_sidecar_files(targetfile):
+            logger.debug(" ... reading existing metadata from {sidecar.name}")
             if is_collection:
-                collection_extra = store.get_for_collection(sidecar_file)
-
-                logger.debug("Read collection metadata: %s", collection_extra)
-
-                if self.selected_path in collection_extra:
-                    extra = collection_extra.pop(targetfile)
-                    del extra[shared.IS_RECURSIVE]
-                else:
-                    extra = shared.CacheEntry(self.selected_path)
-                logger.debug("Extra for %s is: %s", targetfile.name, extra)
-
+                data = stores.get_for_collection(sidecar)
+                if targetfile.parent in data:
+                    metadata.update(data[targetfile.parent])
+                if targetfile in data:
+                    metadata.update(data[targetfile])
+                    collections_to_touch.add(sidecar)
             else:
-                logger.debug(" ... reading metadata from %s", sidecar_file)
-                extra = store.get(sidecar_file)
-                extra.path = targetfile
-                # remove control data
-                del extra[shared.IS_RECURSIVE]
-        else:
-            logger.debug(" ... no sidecar file there yet")
-            if is_collection:
-                collection_extra = {}
-            extra = shared.CacheEntry(targetfile)
+                if sidecar.suffix in preferred_sidecar_exts:
+                    existing_sidecars.append(sidecar)
+                metadata.update(stores.get(sidecar))
 
-        logger.debug(f" ... {extra.metadata}")
+        # sort existing direct sidecars by preference of the user
+        # the most preferred one is the first in the list
+        existing_sidecars.sort(key=lambda s: preferred_sidecar_exts.index(s.suffix))
 
-        # apply all changes to the extra metadata
+        # 2. apply changes in order to metadata
         for change in self.expand_changes():
             logger.debug(f" ... processing {change}")
             prefix = shared.EXTRA
@@ -440,69 +448,89 @@ class EditorPanel(ListPanel):
                 prefix = change.prefix + '.'
 
             if isinstance(change, Change):
-                values = extra.pop(prefix + change.tag)
+                values = metadata.pop(prefix + change.tag)
 
                 applied = False
                 for value in values:
                     if value == change.old_value and not applied:
-                        extra.add(prefix + change.tag, change.new_value)
+                        metadata.add(prefix + change.tag, change.new_value)
                         applied = True
                     else:
-                        extra.add(prefix + change.tag, value)
+                        metadata.add(prefix + change.tag, value)
 
                 if not applied:
                     logger.info("Change to %s is actually an insert because of different sources",
                                 prefix+change.tag)
-                    extra.add(prefix + change.tag, change.new_value)
+                    metadata.add(prefix + change.tag, change.new_value)
 
-            elif isinstance(change, Insert):
-                extra.add(prefix + change.tag, change.value)
+            elif isinstance(change, (Insert, AddToAll)):
+                if (prefix + change.tag, change.value) not in metadata:
+                    metadata.add(prefix + change.tag, change.value)
 
             elif isinstance(change, Delete):
-                values = extra.pop(prefix + change.tag)
+                values = metadata.pop(prefix + change.tag)
                 if len(values) == 0:
                     logger.warning(f"Skipping deletion of {prefix + change.tag}: not found")
                     continue
                 if change.value in values:
                     values.remove(change.value)
                 for value in values:
-                    extra.add(prefix + change.tag, value)
+                    metadata.add(prefix + change.tag, value)
 
-        # 'store' will not remove the 'extra.' prefix per tag,
-        # so we have to do that here
-        logger.debug("Writing new metadata: %s to %s",
-                     extra.metadata, sidecar_file)
-
-        # save the extra metadata to the sidecar file
-        if is_collection:
-            assert collection_extra is not None
-            collection_extra[targetfile.name] = extra
-            logger.debug(f"Updating collection sidecar to {collection_extra}")
-            logger.debug("%s", extra.metadata)
-            store.store(list(collection_extra.values()), sidecar_file)
+        # 3. save metadata to existing or preferred direct sidecar
+        if len(existing_sidecars) == 0:
+            sidecar, _, _ = miconf.resolve_sidecar_for(targetfile, allow_collections=False)
         else:
-            store.store(extra, sidecar_file)
+            sidecar = existing_sidecars.pop(0)
+
+        stores.store(metadata, sidecar)
+
+        # 4. remove *other* direct sidecar files
+        for sidecar in existing_sidecars:
+            sidecar.unlink()
+
+        # 5. remove file-specific metadata from all collection sidecars
+        for sidecar in collections_to_touch:
+            data = stores.get_for_collection(sidecar)
+            if targetfile not in data:
+                continue
+            del data[targetfile]
+            stores.store(list(data.values()), sidecar)
 
     def reload(self):
-        logger.debug("Reloading %s", self.item)
+        logger.debug("Reloading %s", [p.name for p in self.files])
 
-        metadata = [entry
-                    for entry in self.app.cache.get(self.item)
-                    if entry.path == self.item]
+        self.metadata = shared.CacheEntry(self.files[0])
+        self.meta_per_file.clear()
 
-        if len(metadata) == 0 and self.auto_index_if_not_in_cache:
+        reindexed_some = False
+        for target in self.files:
+            metadata = [entry
+                        for entry in self.app.cache.get(target)
+                        if entry.path == target]
+
+            if len(metadata) == 0 and self.auto_index_if_not_in_cache:
+                index_cmd = command.resolve_command('index')
+                if index_cmd is not None:
+                    self.run_blocking(index_cmd.run_indexers,
+                                      self.app,
+                                      self,
+                                      target,
+                                      True)
+                    reindexed_some = True
+                continue
+
+            if len(metadata) > 0:
+                self.meta_per_file[target] = metadata[0]
+                self.metadata.update(metadata[0])
+            else:
+                self.meta_per_file[target] = shared.CacheEntry(target)
+
+        if reindexed_some:
             self.auto_index_if_not_in_cache = False
-            index_cmd = command.resolve_command('index')
-            if index_cmd is not None:
-                self.run_blocking(index_cmd.run_indexers, self.app, self, self.item)
-                return
+            return self.reload()
 
-        if len(metadata) == 0:
-            self.metadata = shared.CacheEntry(self.item)
-        else:
-            self.metadata = metadata[0]
-
-        logger.debug("Reload: %s (%s)", self.metadata.keys(), self.app.cache)
+        logger.debug(f"meta per file: {self.meta_per_file}")
 
         self.rebuild_items()
         self.rebuild_columns()
@@ -529,6 +557,10 @@ class EditorPanel(ListPanel):
             return
 
         keys = list(set(self.metadata.keys()))
+        if len(self.files) > 1:
+            keys = [k for k in keys
+                    if k.startswith(shared.EXTRA)]
+            keys.append(shared.CacheEntry.FILENAME)
         keys.sort(key=lambda k: [not k.startswith(shared.EXTRA), '.' in k, k.lower()])
         logger.debug("editor uses these keys: %s", keys)
 
@@ -538,9 +570,25 @@ class EditorPanel(ListPanel):
             prefix = 'general'
             if '.' in key:
                 prefix, displaykey = key.split('.', 1)
-            for value in self.metadata[key]:
-                logger.debug(" ... adding %s: %s", key, value)
-                self.unchanged_items.append(Line(prefix, prefix, displaykey, value))
+
+            if key == shared.CacheEntry.FILENAME:
+                values = [f.name for f in self.files]
+                values.sort()
+            else:
+                values = self.metadata[key]
+
+            for value in values:
+                # count how often this (key, value) appears in all metadata
+                partial = False
+                if len(self.files) > 1 and prefix == shared.EXTRA[:-1]:
+                    occurrences = [f for f in self.files
+                                   if (key, value) in self.meta_per_file[f]]
+                    partial = len(occurrences) != len(self.files)
+                self.unchanged_items.append(Line(prefix,
+                                                 prefix,
+                                                 displaykey,
+                                                 value,
+                                                 partial))
 
         # apply all changes in order to the point where we are
         self.items = self.unchanged_items[:]
@@ -550,16 +598,20 @@ class EditorPanel(ListPanel):
                 self.items[change.index] = Line(original.group,
                                                 original.prefix,
                                                 original.tag,
-                                                change.new_value)
+                                                change.new_value,
+                                                False)
             elif isinstance(change, Insert):
                 group = change.prefix
                 if len(group) == 0:
                     group = 'general'
-                self.items.append(Line(group, change.prefix, change.tag, change.value))
+                self.items.append(Line(group, change.prefix, change.tag, change.value, False))
+            elif isinstance(change, AddToAll):
+                self.items[change.index].partial = False
             elif isinstance(change, Delete):
                 self.items = self.items[:change.index] + self.items[change.index+1:]
 
             self.items = [i for i in self.items if isinstance(i, Line)]
+            # headers must be added in each step, otherwise the indeces are off
             self.items += [Header(g)
                            for g in {i.group for i in self.items}]
 
@@ -572,7 +624,6 @@ class EditorPanel(ListPanel):
         else:
             for colidx, text in enumerate([item.tag, item.value]):
                 self.win.addstr(y, x, " "*self.columns[colidx])
-                # maxlen = self.dim[1]-x-2
                 maxlen = self.columns[colidx]
                 if colidx == 0:
                     maxlen -= self.SPACING
@@ -580,6 +631,7 @@ class EditorPanel(ListPanel):
                 if text is None:
                     text = ''
                 if colidx == 1 and is_selected and self.editor is not None:
+                    self.win.addstr(y, x-1, self.editor.background)
                     self.editor.paint()
                 else:
                     # make it a human-readable string
@@ -601,6 +653,10 @@ class EditorPanel(ListPanel):
                         shortened = shortened[:-1-len(icon)] + ' ' + icon
 
                     self.win.addstr(y, x, shortened[:maxlen])
+                    # display indicator if a value is partially present
+                    if item.partial and colidx == 1:
+                        self.win.addstr(y, x-2, '*')
+
                 x += self.columns[colidx]
 
     def configuration_changed(self, name=None):
@@ -654,6 +710,17 @@ class EditorPanel(ListPanel):
 
         self.paint(True)
 
+    def add_tag_to_all(self, line):
+        if self.editor is not None:
+            self.cancel_edit()
+
+        self.changed(AddToAll(self.items.index(line),
+                              line.prefix,
+                              line.tag,
+                              line.value))
+
+        self.paint(True)
+
     def remove_tag(self, line):
         if line not in self.items or isinstance(line, str):
             return
@@ -696,16 +763,19 @@ class EditMetadata(command.Command):
     NAME = 'edit-metadata'
     ACCEPT_IN = (DocPanel, FilePanel, DetailPanel)
 
-    def execute(self, context):
+    def execute(self, context, *items):
         if context.panel.is_busy:
             return
-        item = context.panel.selected_path
 
-        if not item.is_file():
-            context.application.error(f"{item.stem} is not a file")
+        if len(items) == 0:
+            items = context.panel.selected_paths
+
+        if any(not i.is_file() for i in items):
+            item = [i for i in items if not i.is_file()]
+            context.application.error(f"{item[0].name} is not a file")
             return
 
-        panel = EditorPanel(item, context.application)
+        panel = EditorPanel(items, context.application)
         context.application.add_panel(panel)
         context.application.activate_panel(panel)
 
@@ -724,7 +794,12 @@ class EnterEditMode(command.Command):
         if not isinstance(item, Line):
             return
 
-        text = str(item.value.raw_value)
+        text = str(item.value)
+        if isinstance(item.value, shared.MetadataValue):
+            text = str(item.value.raw_value)
+
+        # allow editing even for non-extra tags, if they have
+        # a multiline value (which is not properly displayed here)
         if item.prefix == shared.EXTRA[:-1] or \
            ('\n' in text or '\r' in text):
             context.panel.start_edit()
@@ -766,6 +841,20 @@ class AddTagCommand(command.Command):
         context.panel.add_tag(name)
 
 
+@command.simple_command("add-tag-to-all", accept_in=(EditorPanel,))
+def add_tag_to_all_command(context):
+    """Add this metadata tag to all edited files"""
+    if context.panel.is_busy:
+        return
+
+    item = context.panel.selected_line
+    if not item.partial:
+        context.application.info("This tag is already set for all edited files")
+        return
+
+    context.panel.add_tag_to_all(item)
+
+
 @command.registered_command
 class AddValueForAttribute(command.Command):
     """Add a new metadata value for this field"""
