@@ -1,6 +1,7 @@
 import pathlib
 import curses
 import os
+import re
 import shutil
 import threading
 import subprocess
@@ -30,6 +31,9 @@ from metaindexmanager.utils import logger
 from metaindexmanager.panel import ListPanel, register
 
 
+ENV_VAR_RE = re.compile(r'\$[A-Z_][A-Z0-9_]*')
+
+
 class Line:
     def __init__(self, panel, path, stat=None):
         self.panel = panel
@@ -233,7 +237,7 @@ class FilePanel(ListPanel):
             self.observer = None
         if self.path != path:
             if self.path is not None:
-                self.cursor_cache[self.path] = self.cursor
+                self.cursor_cache[self.path] = (self.cursor, self.selected_path)
             self.previous_path = self.path
             self.path = path
             self.offset = 0
@@ -250,7 +254,12 @@ class FilePanel(ListPanel):
                          (self.show_sidecar_files or not is_sidecar(i.path))]
 
         if self.path in self.cursor_cache:
-            self.cursor = self.cursor_cache[self.path]
+            self.cursor, to_select = self.cursor_cache[self.path]
+            to_select = [idx
+                         for idx, item in enumerate(self.items)
+                         if item.path == to_select]
+            if len(to_select) > 0:
+                self.cursor = to_select[0]
 
         elif self.previous_path is not None and self.path == self.previous_path.parent:
             self.cursor = 0
@@ -283,7 +292,7 @@ class FilePanel(ListPanel):
 
     @property
     def selected_path(self):
-        if self.selected_item is None:
+        if self.items is None or self.selected_item is None:
             return None
         return self.selected_item.fullpath
 
@@ -644,7 +653,9 @@ class FilePanel(ListPanel):
             self.observer.wake_up()
 
     def reload(self):
-        self.change_path(self.path)
+        if self.path is not None:
+            self.cursor_cache[self.path] = (self.cursor, self.selected_path)
+            self.change_path(self.path)
 
     def paint(self, clear=False):
         if self.items is None:
@@ -702,7 +713,7 @@ class FilePanel(ListPanel):
     def handle_scrolling_key(self, key):
         handled, must_repaint, must_clear = super().handle_scrolling_key(key)
         if handled:
-            self.cursor_cache[self.path] = self.cursor
+            self.cursor_cache[self.path] = (self.cursor, self.selected_path)
         return handled, must_repaint, must_clear
 
     def line_matches_find(self, cursor):
@@ -807,7 +818,7 @@ def duration_as_str(duration):
     days = seconds // 86400
     hours = (seconds % 86400) // 3600
     minutes = (seconds % 3600) // 60
-    
+
     result = ''
     if days > 0:
         result += f'{days}d'
@@ -918,11 +929,13 @@ class ChangeDirectory(command.Command):
             return
 
         if not isinstance(target, pathlib.Path):
-            try:
-                target = pathlib.Path(target).expanduser().resolve()
-            except:
-                context.application.error(f"{target} is not a valid path")
-                return
+            target = pathlib.Path(target).expanduser()
+
+        parts = [expand_env_variables(p)
+                 for p in target.parts]
+        if parts[0] == '/' and parts[1] == '/':
+            parts.pop(0)
+        target = pathlib.Path(os.sep.join(parts)).resolve()
 
         if not target.is_dir():
             context.application.error(f"{target} does not exist")
@@ -1011,7 +1024,10 @@ def shell_command(context, *args):
     scr = context.application.screen
     with ShellContext(scr, True):
         try:
-            subprocess.run(" ".join(args), shell=True, check=False)
+            subprocess.run(" ".join(args),
+                           cwd=context.panel.path,
+                           shell=True,
+                           check=False)
         except (KeyboardInterrupt,):
             pass
 
@@ -1045,3 +1061,11 @@ def launch_command(context, *args):
                          stderr=subprocess.DEVNULL)
     except OSError:
         pass
+
+
+def expand_env_variables(arg):
+    for match_ in ENV_VAR_RE.finditer(arg):
+        arg = arg[:match_.start()] \
+            + os.getenv(match_[0][1:], match_[0]) \
+            + arg[match_.end()+1:]
+    return arg
