@@ -165,6 +165,7 @@ class Application(cursedspace.Application):
             (shared.ANY_SCOPE, ('y', '0'), ('clear-clipboard',)),
             (FilePanel.SCOPE, ('z', 'h'), ('toggle-hidden',)),
             (FilePanel.SCOPE, ('z', 'm'), ('toggle-sidecar',)),
+            (DocPanel.SCOPE, ('z', 'o'), ('toggle-show-offline',)),
             (shared.ANY_SCOPE, ('z', 's'), ('::layout horizontal', 'Split layout')),
             (shared.ANY_SCOPE, ('z', 't'), ('::layout tabbed', 'Tabbed layout')),
             (shared.ANY_SCOPE, (':', ), ('enter-command',)),
@@ -199,6 +200,7 @@ class Application(cursedspace.Application):
             (EditorPanel.SCOPE, ('o',), ('add-value',)),
             (EditorPanel.SCOPE, ('c',), ('replace-value',)),
             (EditorPanel.SCOPE, ('a',), (':add-tag', 'Add a new tag')),
+            (EditorPanel.SCOPE, ('*',), ('add-tag-to-all',)),
             (EditorPanel.SCOPE, ('d', 'd'), ('del-tag',)),
             (EditorPanel.SCOPE, ('u',), ('undo-change',)),
             (EditorPanel.SCOPE, ('U',), ('undo-all-changes',)),
@@ -388,20 +390,30 @@ class Application(cursedspace.Application):
                             self.current_panel.handle_key(seq)
                     elif commandname.startswith(':') and not commandname.startswith('::'):
                         rawprefill = shlex.split(commandname[1:])
-                        prefill = shlex.join([rawprefill[0]]
-                                             + sum([expand_part(self, part)
-                                                    for part in rawprefill[1:]], start=[]))
-                        if len(rawprefill) == 1:
-                            prefill += " "
-                        self.execute_command("enter-command", prefill)
+                        try:
+                            prefill = shlex.join([rawprefill[0]]
+                                                 + sum([expand_part(self, part)
+                                                        for part in rawprefill[1:]], start=[]))
+                        except RuntimeError as exc:
+                            self.error(f"Could not run '{rawprefill[0]}': {exc}")
+                            prefill = None
+
+                        if prefill is not None:
+                            if len(rawprefill) == 1:
+                                prefill += " "
+                            self.execute_command("enter-command", prefill)
                     else:
                         args = None
                         if commandname.startswith('::'):
                             args = shlex.split(commandname[2:])
                             commandname = args.pop(0)
-                            args = sum([expand_part(self, part)
-                                        for part in args],
-                                       start=[])
+                            try:
+                                args = sum([expand_part(self, part)
+                                            for part in args],
+                                           start=[])
+                            except RuntimeError as exc:
+                                self.error(f"Could not execute {commandname}: {exc}")
+                                commandname = None
 
                         command = resolve_command(commandname)
                         if command is not None and \
@@ -1073,7 +1085,8 @@ def run():
     logging.getLogger().addHandler(logger.handler)
 
     if args.log_level.lower() in ['debug']:
-        metaindex.logger.setup(level=logging.DEBUG, filename=args.log_file.parent / 'metaindex.log')
+        metaindex.logger.setup(level=logging.DEBUG,
+                               filename=args.log_file.parent / 'metaindex.log')
     else:
         metaindex.logger.setup(logging.ERROR, logger.handler)
 
