@@ -4,6 +4,7 @@ PROGRAMNAME = 'metaindexmanager'
 
 ANY_SCOPE = 'any'
 ALL_SCOPE = 'all'
+DICTIONARY_SCOPE = 'dictionary'
 
 PREF_BORDER = 'border'
 DEFAULT_BORDER = 'full'
@@ -12,3 +13,6 @@ BORDER_MINIMAL = 'minimal'
 
 PREF_DEFAULT_PANEL = 'default-panel'
 DEFAULT_DEFAULT_PANEL = 'documents'
+
+PREF_FIND_CASE_SENSITIVE = 'find-case-sensitive'
+DEFAULT_FIND_CASE_SENSITIVE = 'no'
